# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the firwin function

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import unittest
import numpy as np
import numpy.linalg as LA

from ltfatpy.sigproc.firwin import firwin
from ltfatpy.fourier.isevenfunction import isevenfunction
from ltfatpy.sigproc.fir2long import fir2long


class TestSigprocFirwin(unittest.TestCase):
    """ Unittest class for Sigproc firwin"""
    digit = 9

    def setUp(self):
        pass

    def tearDown(self):
        print('Test done')

    def test_default(self):
        """  This test script verifies the firwin properties """
        allwins = ('hann', 'tria', 'sine', 'sqrttria', 'itersine', 'square',
                   'hamming', 'blackman', 'nuttall', 'nuttall10', 'nuttall01',
                   'nuttall20', 'nuttall11', 'nuttall02', 'nuttall30',
                   'nuttall21', 'nuttall03', 'blackman2')

        for L in (18, 19.0, 20, 21):
            for centype in ('wp', 'hp'):
                for ii in range(len(allwins)):
                    winname = allwins[ii]
                    (g, info) = firwin(winname, L, centering=centype)
                    L = int(L)
                    res = 1 - isevenfunction(fir2long(g, 2*L),
                                             centering=centype)
                    s = "SYMM {0} {1} L: {2:d} {3:0.5g}".format(winname,
                                                                centype, L,
                                                                res)
                    np.testing.assert_almost_equal(res, 0, type(self).digit,
                                                   err_msg=s)
                    if centype == 'wp':
                        res = 1 - g[0]
                        s = "PEAK {0} {1} L: {2:d} {3:0.5g}".format(winname,
                                                                    centype,
                                                                    L, res)
                        np.testing.assert_almost_equal(res, 0,
                                                       type(self).digit,
                                                       err_msg=s)
                    if L % 2 == 0:
                        if info['ispu']:
                            gpu = g + np.fft.fftshift(g)
                            res = LA.norm(gpu-gpu[0] * np.ones(L))
                            s = "SYMM {0} {1} L: {2:d} {3:0.5g}".format(
                                                                winname,
                                                                centype, L,
                                                                res)
                            np.testing.assert_almost_equal(res, 0,
                                                           type(self).digit,
                                                           err_msg=s)
                        if info['issqpu']:
                            gpu = g**2 + np.fft.fftshift(g**2)
                            res = LA.norm(gpu - gpu[0] * np.ones(L))
                            s = "SYMM {0} {1} L: {2:d} {3:0.5g}".format(
                                                                winname,
                                                                centype, L,
                                                                res)
                            np.testing.assert_almost_equal(res, 0,
                                                           type(self).digit,
                                                           err_msg=s)

    def test_exceptions(self):
        """  This test script verifies the firwin exceptions """
        self.assertRaises(TypeError, firwin, 2, 10)
        self.assertRaises(TypeError, firwin, 'hann', '10')
        self.assertRaises(ValueError, firwin, 'toto', 10)
        self.assertRaises(ValueError, firwin, 'hann', 10, np.ones(9))

    def test_param(self):
        np.testing.assert_array_equal(firwin('hann', 10, shift=0.5)[0],
                                      firwin('hann', 10, centering='hp')[0])
        np.testing.assert_array_equal(firwin('hann', 20, taper=0.5)[0],
                                      np.hstack((np.ones(5),
                                                 firwin('hann', 10)[0],
                                                 np.ones(5))))
        self.assertFalse(firwin('hann', 0)[0])
        np.testing.assert_array_equal(firwin('hann', 10, taper=0)[0],
                                      np.ones(10))
        np.testing.assert_array_equal(firwin('hann', 10, taper=0.5)[0],
                                      np.hstack((np.ones(2),
                                                 firwin('hann', 5,
                                                        shift=0.5)[0],
                                                 np.ones(3))))
        np.testing.assert_array_equal(
                        firwin('hann', 11, taper=0.5, shift=0.5)[0],
                        np.hstack((np.ones(2), firwin('hann', 6, shift=1)[0],
                                   np.ones(3)))
                        )
        np.testing.assert_array_equal(firwin('sqrttria', 10)[0],
                                      np.sqrt(firwin('tria', 10)[0]))
        x = np.arange(10)
        g, _ = firwin('hann', x=x, norm='1')
        gres = np.zeros(10, dtype='f4')
        gres[0] = 1.0
        np.testing.assert_array_equal(g, gres)

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestSigprocFirwin)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of spectrogram plotting

Ported from ltfat_2.1.0/gabor/sgram.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.gabor.plotdgt import plotdgt
from ltfatpy.gabor.plotdgtreal import plotdgtreal
from ltfatpy.gabor.gabimagepars import gabimagepars
from ltfatpy.fourier.fftresample import fftresample
from ltfatpy.sigproc.largestr import largestr
from ltfatpy.gabor.dgt import dgt
from ltfatpy.gabor.dgtreal import dgtreal


def sgram(f, fs=None, tfr=1., wlen=None, nf=None, thr=None, fmax=None,
          xres=800, yres=600, norm='2', **kwargs):
    """Spectrogram

    - Input:

    :param numpy.ndarray f: Analyzed signal
    :param float fs: Sampling rate in Hz of the analyzed signal
    :param float tfr: Set the ratio of frequency resolution to time resolution.
        A value ``tfr = 1.0`` is the default. Setting ``tfr > 1.0`` will give
        better frequency resolution at the expense of a worse time resolution.
        A value of ``0.0 < tfr < 1.0`` will do the opposite.
    :param int wlen: Window length. Specifies the length of the window
        measured in samples. See help of :func:`~ltfatpy.fourier.pgauss` on the
        exact details of the window length (parameter **width**).
    :param bool nf: If ``True``, display negative frequencies, with the
        zero-frequency centered in the middle. For real signals, this will just
        mirror the upper half plane. This is standard for complex signals.
    :param float thr: Keep only the largest fraction **thr** of the
        coefficients, and set the rest to zero.
    :param float fmax: Display **fmax** as the highest frequency. Default value
        of ``None`` means to use the Nyquist frequency.
    :param int xres: Approximate number of pixels along x-axis / time
    :param int yres: Approximate number of pixels along y-axis / frequency
    :param string norm: Window normalization. See parameter **norm** in the
        help of :func:`~ltfatpy.fourier.pgauss`.
    :param `**kwargs`: ``sgram`` supports all the optional parameters of
        :func:`~ltfatpy.gabor.tfplot.tfplot`. Please see the help of
        :func:`~ltfatpy.gabor.tfplot.tfplot` for an exhaustive list.

    - Output parameters:

    :returns: The image to be displayed as a matrix. Use this in conjunction
        with imwrite etc. These coefficients are *only* intended to be used by
        post-processing image tools. Numerical Gabor signal analysis and
        synthesis should *always* be done using the
        :func:`~ltfatpy.gabor.dgt.dgt`, :func:`~ltfatpy.gabor.idgt.idgt`,
        :func:`~ltfatpy.gabor.dgtreal.dgtreal` and
        :func:`~ltfatpy.gabor.idgtreal.idgtreal` functions.
    :rtype: numpy.ndarray

    ``sgram(f)`` plots a spectrogram of **f** using a Discrete Gabor Transform
    (:func:`~ltfatpy.gabor.dgt.dgt`).

    - Examples:

        The :func:`~ltfatpy.signals.greasy.greasy` signal is sampled using a
        sampling rate of 16 kHz. To  display a spectrogram of
        :func:`~ltfatpy.signals.greasy.greasy` with a dynamic range of 90 dB,
        use:

        >>> from ltfatpy import sgram, greasy
        >>> from matplotlib.pyplot import show
        >>> _ = sgram(greasy()[0], 16000., dynrange=90)
        >>> show()

        To create a spectrogram with a window length of 20 ms (which is
        typically used in speech analysis) use :

        >>> from ltfatpy import sgram, greasy
        >>> from matplotlib.pyplot import show
        >>> fs = 16000.
        >>> _ = sgram(greasy()[0], fs, dynrange=90, wlen=round(20./1000.*fs))
        >>> show()

    .. image:: images/sgram_1.png
       :width: 600px
       :alt: spectrogram image a dynamic range of 90 dB
       :align: center

    .. image:: images/sgram_2.png
       :width: 600px
       :alt: spectrogram image with a window length of 20 ms
       :align: center

    .. seealso:: :func:`~ltfatpy.gabor.dgt.dgt`,
        :func:`~ltfatpy.gabor.dgtreal.dgtreal`

    """

    if not isinstance(f, np.ndarray):
        raise TypeError('f must be a 1D numpy.ndarray')

    if f.ndim > 1:
        raise ValueError('Input must be a vector.')

    # NOTE: The Octave code of this function in LTFAT 2.1.0 contains the
    # following comment and code:
    #
    # Override the setting from tfplot, because SGRAM does not support the
    # 'dbsq' setting (it does not make sense).
    # definput.flags.log={'db','lin'};
    #
    # But this seems to have no practical effect as we can still run tfplot
    # with the dbsq option in Octave (and all the other normalization options).
    # So to have  similar results than the Octave function in this Python port
    # we don't limit the possible values for normalization.

    if nf is None:
        if np.isrealobj(f):
            nf = False
        else:
            nf = True

    # Downsample
    if fmax:
        if fs:
            resamp = fmax*2./fs
        else:
            resamp = fmax*2./f.shape[0]

        f = fftresample(f, int(round(f.shape[0] * resamp)))
        fs = 2. * fmax

    Ls = f.shape[0]

    if not nf:
        yres = 2*yres

    try:
        a, M, L, N, Ndisp = gabimagepars(Ls, xres, yres)

    except ValueError:
        raise ValueError(
            'The signal is too long. sgram cannot visualize all the '
            'details.\nTry a shorter signal or increase the image resolution '
            'by calling:\n\nsgram(..., xres=xres, yres=yres)\n\n'
            'for larger values of xres and yres.\nThe current values are:\n  '
            'xres = {0}\n  yres = {1}'.format(xres, yres))

    except:
        raise

    # Set an explicit window length, if this was specified.
    if wlen:
        tfr = wlen**2 / L

    g = {'name': 'gauss', 'tfr': tfr, 'norm': norm}

    if nf:
        coef = np.abs(dgt(f, g, a, M)[0])
    else:
        coef = np.abs(dgtreal(f, g, a, M)[0])

    # Cut away zero-extension.
    coef = coef[:, 0:Ndisp]

    if thr:
        # keep only the largest coefficients.
        coef = largestr(coef, thr)[0]

    if nf:
        coef = plotdgt(coef, a, fs=fs, **kwargs)
    else:
        coef = plotdgtreal(coef, a, M, fs=fs, **kwargs)

    return coef

if __name__ == '__main__':  # pragma: no cover
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of fixed ratio of largest coefficients extraction

Ported from ltfat_2.1.0/sigproc/largestr.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

from ltfatpy.sigproc.largestn import largestn


def largestr(xi, p, thresh_type='hard'):
    """Keep fixed ratio of largest coefficients

    - Usage:

        | ``(xo, Nout) = largestr(xi, p)``
        | ``(xo, Nout) = largestr(xi, p, thresh_type)``

    - Input parameters:

    :param numpy.ndarray xi: Input array
    :param float p: Ratio of kept coefficients
    :param str thresh_type: Optional flag specifying the type of thresholding
        (see possible values below)

    - Output parameters:

    :returns: ``(xo, Nout)``
    :rtype: tuple

    :var numpy.ndarray xo: Array of the same shape as **xi** keeping
        the fraction **p** of the coefficients with the largest magnitude
    :var int Nout: Number of coefficients kept

    .. note::
        If the function is used on coefficients coming from a redundant
        transform or from a transform where the input signal was padded, the
        coefficient array will be larger than the original input signal.
        Therefore, the number of coefficients kept might be higher than
        expected.

    The parameter **thresh_type** can take the following values:
        ============ ======================================================
        ``'hard'``   Perform hard thresholding. This is the default.

        ``'wiener'`` Perform empirical Wiener shrinkage. This is in between
                     soft and hard thresholding.

        ``'soft'``   Perform soft thresholding.
        ============ ======================================================

    .. note::
        If soft- or Wiener thresholding is selected, one less
        coefficient will actually be returned. This is caused by that
        coefficient being set to zero.

    .. seealso::
        :func:`~ltfatpy.sigproc.largestn.largestn`

    - References:
        :cite:`ma98`
    """

    if not isinstance(p, float):
        raise TypeError('p must be a float.')

    N = int(round(xi.size * p))

    xo, Nout = largestn(xi, N, thresh_type)

    return (xo, Nout)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the groupthresh function

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import unittest
import numpy as np
from numpy.testing import assert_array_equal
from copy import deepcopy

from ltfatpy.sigproc.groupthresh import groupthresh
from ltfatpy.tests.datasets.read_ref_mat import read_ref_mat
from ltfatpy.tests.datasets.get_dataset_path import get_dataset_path

# NOTE: The reference values used in the tests correspond to results
# obtained with Octave using ltfat 2.1.0


class TestGroupthresh(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print('\nStart TestGroupthresh')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_known(self):
        """Checking groupthresh on some known results taken from Octave
        """
        filename = get_dataset_path('groupthresh_ref.mat')
        data = read_ref_mat(filename)
        for inputs, outputs in data:
            # lamb is incorreclty converted to an int for certain values by
            # read_ref_mat, so we convert it back to float as needed by
            # groupthresh
            inputs['lamb'] = float(inputs['lamb'])
            out = groupthresh(**inputs)
            msg = ('Wrong value in output of groupthresh with inputs ' +
                   str(inputs))
            assert_array_equal(out, outputs[0], msg)

    def test_shape(self):
        """Check that input and output shapes match
        """
        shape = (4, 3)
        thresh_types = ('hard', 'wiener', 'soft')
        group_types = ('group', 'elite')
        dims = (0, 1)
        inputs = {}
        inputs['xi'] = np.random.random(shape)
        inputs['lamb'] = 0.5
        for dim in dims:
            inputs['dim'] = dim
            for group_type in group_types:
                inputs['group_type'] = group_type
                for thresh_type in thresh_types:
                    inputs['thresh_type'] = thresh_type
                    xo = groupthresh(**inputs)
                    msg = ('Wrong shape in output xo of groupthresh with '
                           'inputs ' + str(inputs))
                    self.assertEqual(xo.shape, shape, msg)

    def test_default_param(self):
        """Check that the default value for thresh_type is right
        """
        inputs_def = {}
        inputs_def['xi'] = np.random.random((4, 3))
        inputs_def['lamb'] = 0.5
        xo_def = groupthresh(**inputs_def)

        inputs_group = deepcopy(inputs_def)
        inputs_group['dim'] = 1
        xo_group = groupthresh(**inputs_group)
        msg = ('Wrong default value for dim in groupthresh when '
               'comparing results with inputs ' + str(inputs_def) + ' and ' +
               str(inputs_group))
        assert_array_equal(xo_def, xo_group, msg)

        inputs_group = deepcopy(inputs_def)
        inputs_group['group_type'] = 'group'
        xo_group = groupthresh(**inputs_group)
        msg = ('Wrong default value for group_type in groupthresh when '
               'comparing results with inputs ' + str(inputs_def) + ' and ' +
               str(inputs_group))
        assert_array_equal(xo_def, xo_group, msg)

        inputs_hard = deepcopy(inputs_def)
        inputs_hard['thresh_type'] = 'soft'
        xo_hard = groupthresh(**inputs_hard)
        msg = ('Wrong default value for thresh_type in groupthresh when '
               'comparing results with inputs ' + str(inputs_def) + ' and ' +
               str(inputs_hard))
        assert_array_equal(xo_def, xo_hard, msg)

    def test_type(self):
        """Check that the type for lamb is correctly checked
        """
        xi = np.ones((4, 3))
        # test with lamb of wrong type (str, when float is expected)
        lamb = 'test'
        self.assertRaises(TypeError, groupthresh, xi, lamb)
        # test with lamb of wrong type (int, when float is expected)
        lamb = 1
        self.assertRaises(TypeError, groupthresh, xi, lamb)

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGroupthresh)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of order parameter growing

Ported from ltfat_2.1.0/comp/assert_groworder.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np


def assert_groworder(order):
    """Grow the order parameter

    - Input parameter:

    :param tuple order: Order indices obtained with
        :func:`~ltfatpy.comp.assert_sigreshape_pre.assert_sigreshape_pre`

    - Ouput parameter:

    :return: Order indices
    :rtype: tuple

    ``assert_groworder`` is meant to be used in conjunction with
    :func:`~ltfatpy.comp.assert_sigreshape_pre.assert_sigreshape_pre` and
    :func:`~ltfatpy.comp.assert_sigreshape_post.assert_sigreshape_post`. It is
    used to modify the **order** parameter in between calls in order to expand
    the processed dimension by ``1``, i.e. for use in a routine that creates 2D
    output from 1D input, for instance in :func:`~ltfatpy.gabor.dgt.dgt` or
    :func:`filterbank`.
    """

    if len(order) > 1:
        # We only need to handle the non-trivial order, where dim>1
        p = order[0]
        # Shift orders higher that the working dimension by 1, to make room for
        # the new dimension, but leave lower dimensions untouched.
        ordernp = np.asarray(order)
        ordernp[ordernp > p] = ordernp[ordernp > p]+1
        return (p, p+1) + tuple(ordernp[1:])
    return order



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the fftindex function

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import unittest
import numpy as np
from numpy.testing import assert_array_equal

from ltfatpy.fourier.fftindex import fftindex


class TestFftindex(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print('\nStart TestFftindex')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_shape(self):
        """Check that the output has the expected shape
        """
        N_val = (13, 14, 31, 32)
        nyquistzero_val = (False, True)
        inputs = {}
        for N in N_val:
            inputs['N'] = N
            for nyquistzero in nyquistzero_val:
                inputs['nyquistzero'] = nyquistzero
                n = fftindex(**inputs)
                msg = ('Wrong shape in the ouput of fftindex with inputs ' +
                       str(inputs))
                self.assertEqual(n.shape, (inputs['N'],), msg)

    def test_known(self):
        """Checking fftindex on some known results taken from Octave
        """
        N_val = (5, 6)
        nyquistzero_val = (False, True)
        # Values of n as returned in Octave with ltfat 2.1.0
        n_oct = ((np.array([0, 1, 2, -2, -1]), np.array([0, 1, 2, -2, -1])),
                 (np.array([0, 1, 2, 3, -2, -1]),
                  np.array([0, 1, 2, 0, -2, -1])))
        inputs = {}
        for N, n_refs in zip(N_val, n_oct):
            inputs['N'] = N
            for nyquistzero, n_ref in zip(nyquistzero_val, n_refs):
                inputs['nyquistzero'] = nyquistzero
                msg = 'Wrong output of fftindex with inputs ' + str(inputs)
                assert_array_equal(fftindex(**inputs), n_ref, msg)

    def test_default_param(self):
        """Check that the default value for nyquistzero is right
        """
        inputs_def = {}
        inputs_def['N'] = 6
        inputs_false = {}
        inputs_false['N'] = inputs_def['N']
        inputs_false['nyquistzero'] = False
        n_def = fftindex(**inputs_def)
        n_false = fftindex(**inputs_false)
        msg = ('Wrong default value for nyquistzero in fftindex when '
               'comparing results with inputs ' + str(inputs_def) + ' and ' +
               str(inputs_false))
        assert_array_equal(n_def, n_false, msg)

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestFftindex)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module that cuts long windows to fir

Ported from ltfat_2.1.0/sigproc/long2fir.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.fourier.middlepad import middlepad


def long2fir(g, L, centering='unsymmetric'):
    """Cut LONG window to FIR

    - Usage:

        | ``g = long2fir(g, L)``
        | ``g = long2fir(g, L, 'hp')``

    - Input parameters:

    :param numpy.ndarray g: long window
    :param int L: length of the output FIR window
    :param str centering: 'wp' or 'hp' for whole point even or half point
        even input window. 'unsymmetric' makes no assumption on the symmetry
        of the input data. Default is 'unsymmetric'

    - Output parameters:

    :returns: the FIR window
    :rtype: numpy.ndarray

    ``long2fir(g, L)`` will cut the LONG window **g** to a length **L** FIR
    window by cutting out the middle part. Note that this is a slightly
    different behaviour than :func:`~ltfatpy.fourier.middlepad.middlepad`.

    ``long2fir(g, L, 'wp')`` or ``long2fir(g, L, 'hp')`` does the same
    assuming the input window is a whole-point even or half-point even
    window, respectively.

    .. seealso::  :func:`~ltfatpy.sigproc.fir2long.fir2long`,
        :func:`~ltfatpy.fourier.middlepad.middlepad`
    """
    if not isinstance(g, np.ndarray):
        raise TypeError("g must be a numpy array")
    if L is None:
        raise ValueError('You must specify a way to shorten the window,' +
                         'either by specifying the length or through a flag.')
    W = len(g)
    if W < L:
        raise ValueError('L must be smaller than length of window.')

    # Not translated from Matlab :
    #     if ~isempty(kv.cutrel)
    #     maxval=max(abs(g));
    #     mask=abs(g)>maxval*kv.cutrel;
    #     L=W-2*min(abs(find(mask)-L/2));
    #     end;

    if centering == 'unsymmetric':
        # No assumption on the symmetry of the window.
        if L % 2 == 0:
            # HPE middlepad works the same way as the FIR cutting (e.g. just
            # removing middle points) for even values of L.
            g = middlepad(g, L, centering='hp')
        else:
            # WPE middlepad works the same way as the FIR cutting (e.g. just
            # removing middle points) for odd values of L.
            g = middlepad(g, L)
    elif centering == 'wp':
        g = middlepad(g, L)
        if L % 2 == 0:
            g[L//2] = 0
    elif centering == 'hp':
        g = middlepad(g, L, centering='hp')
        if L % 2 == 1:
            g[int(np.ceil(L/2)) - 1] = 0
    else:
        raise ValueError("centering should take 'hp','wp' or 'unsymmetric'" +
                         " values.")
    return g



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the assert_sigreshape_post function

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import unittest
import random
import numpy as np
from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre as aspre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post as aspost
import functools


class TestAssertSigReshapePost(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print("Start TestAssertSigReshapePost")

    # Called after the tests.
    def tearDown(self):
        print("Test done")

    def test_default(self):
        """Basic usual cases"""
        L = random.randint(5, 200)
        f = np.arange(0, L, dtype=np.float64)
        (g, L, Ls, W, dim, permutedshape, order) = aspre(f)
        fres = aspost(f, dim, permutedshape, order)
        mess = "\nL = {0:d}, Ls = {1:d}, W = {2:d}, dim = {3:d}, g.shape = "
        mess += str(g.shape) + ", f.shape = " + str(f.shape)
        mess += ", fres.shape = " + str(fres.shape)
        mess = mess.format(L, Ls, W, dim)
        np.testing.assert_array_equal(f, fres, mess)

    def test_twodim(self):
        """Many channels signals"""
        shapef = tuple([random.randint(5, 20) for _ in range(2)])
        L = functools.reduce(lambda x, y: x * y, shapef)
        f = np.arange(0, L, dtype=np.complex128)
        f = f.reshape(shapef)
        (g, L, Ls, W, dim, permutedshape, order) = aspre(f)
        fres = aspost(f, dim, permutedshape, order)
        mess = "\nL = {0:d}, Ls = {1:d}, W = {2:d}, dim = {3:d}, g.shape = "
        mess += str(g.shape) + ", f.shape = " + str(f.shape)
        mess += ", fres.shape = " + str(fres.shape)
        mess = mess.format(L, Ls, W, dim)
        np.testing.assert_array_equal(f, fres, mess)


if __name__ == "__main__":
    suite = unittest.TestLoader().loadTestsFromTestCase(TestAssertSigReshapePost)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the read_ref_mat function

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import unittest
from ltfatpy.tests.datasets.read_ref_mat import read_ref_mat
import numpy as np
from numpy.testing import assert_array_equal
from ltfatpy.tests.datasets.get_dataset_path import get_dataset_path


class TestReadRefMat(unittest.TestCase):
    """ Test read_ref_mat.py """

    # Called before the tests.
    def setUp(self):
        print('\nTestReadRefMat')
        self.filename = get_dataset_path('read_ref_mat_test.mat')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_default(self):
        """Test read_ref_mat on some known data
        """

        expected = [({u'fun': u'abs', u'dim': 1, u'var': 1.3},
                     [np.array([3., 2.2]), 1.5]),
                    ({u'fun': u'dgt', u'do_it': True}, [np.array([1.4, 1.2])])]

        res = read_ref_mat(self.filename)

        # NOTE: We would like to compare res and expected, but as they contain
        # some numpy.ndarray, they cannot be direclty compared using == or
        # assertEqual (this leads to the ValueError:
        # The truth value of an array with more than one element is ambiguous.
        # Use a.any() or a.all())
        # So we need to manually loop to compare the results:
        msg = 'Wrong output of read_ref_mat on reference test data'
        self.assertEqual(len(expected), len(res), msg)
        for (in_exp, out_exp), (in_res, out_res) in zip(expected, res):
            self.assertEqual(in_exp, in_res, msg)
            self.assertEqual(len(out_exp), len(out_res), msg)
            for val_exp, val_res in zip(out_exp, out_res):
                if isinstance(val_exp, np.ndarray):
                    assert_array_equal(val_exp, val_res, msg)
                else:
                    self.assertEqual(val_exp, val_res, msg)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestReadRefMat)
    unittest.TextTestRunner(verbosity=2).run(suite)
